#define NDEF "\0"


// define pins
#define lWheelDir     6
#define lWheelStep    7
#define rWheelDir     1
#define rWheelStep    0
#define lSensorEnable NDEF
#define lSensorRead   A6
#define mSensorEnable NDEF
#define mSensorRead   A7
#define rSensorEnable NDEF
#define rSensorRead   A3
#define mouseClock    A0
#define mouseData     A1
#define LEDPin        2
#define speakerPin    NDEF


// define constants
#define ONE_ROUND 1692 // steps for a full rotation
#define SPEED     2000 // minimum delay between steps in us
#define DELAY     200  // delay between modes in ms
#define STEPS     1    // number of steps at once without checking sensors


// create objects
#include "Mouse.h"
Mouse mouse(mouseClock, mouseData, lWheelStep, rWheelStep);

#include <FastLED.h>
#define NUM_LEDS 12
CRGB leds[NUM_LEDS];

#include <SharpDistSensor.h>
#define filterWindow 5
SharpDistSensor mSensor(mSensorRead, filterWindow);


void setup()
{
  // objects
  FastLED.addLeds<WS2812, LEDPin, GRB>(leds, NUM_LEDS);
  mouse.init();

  // declare Pins
  pinMode(lWheelDir,     OUTPUT);
  pinMode(lWheelStep,    OUTPUT);
  pinMode(rWheelDir,     OUTPUT);
  pinMode(rWheelStep,    OUTPUT);
//pinMode(lSensorEnable, OUTPUT);
  pinMode(lSensorRead,   INPUT );
//pinMode(mSensorEnable, OUTPUT);
  pinMode(mSensorRead,   INPUT );
//pinMode(rSensorEnable, OUTPUT);
  pinMode(rSensorRead,   INPUT );

  // initialise Pins
  digitalWrite(lWheelDir,     HIGH);
  digitalWrite(lWheelStep,    LOW );
  digitalWrite(rWheelDir,     LOW );
  digitalWrite(rWheelStep,    LOW );
//digitalWrite(lSensorEnable, HIGH);
//digitalWrite(rSensorEnable, HIGH);
//digitalWrite(mSensorEnable, HIGH);

  delay(DELAY);
  
}


bool testTable(byte mode)
{
  static unsigned short int lSensor, rSensor;

  // take smaller of twoo readings
  lSensor = analogRead(lSensorRead);
  rSensor = analogRead(rSensorRead);
  lSensor = min(lSensor, analogRead(lSensorRead));
  rSensor = min(rSensor, analogRead(rSensorRead));
  
  switch (mode) {
    case 0: // check both sides
      return 512 <= lSensor && 512 <= rSensor;
    break;
    
    case 1: // check left side
      return 512 <= lSensor;
    break;
    
    case 2: // check right side
      return 512 <= rSensor;
    break;
  }
}


byte wheelMode = 0;
unsigned long int lastStep = 0;
void go(byte mode, unsigned short int steps)
{
  if (mode != wheelMode) { // check current direction
    switch (mode) {
      case 0: // forward
        digitalWrite(lWheelDir, HIGH);
        digitalWrite(rWheelDir, LOW);
        wheelMode = 0;
        break;
        
      case 1: // backward
        digitalWrite(lWheelDir, LOW);
        digitalWrite(rWheelDir, HIGH);
        wheelMode = 1;
        break;
        
      case 2: // left
        digitalWrite(lWheelDir, LOW);
        digitalWrite(rWheelDir, LOW);
        wheelMode = 2;
        break;
        
      case 3: // right
        digitalWrite(lWheelDir, HIGH);
        digitalWrite(rWheelDir, HIGH);
        wheelMode = 3;
        break;
        
      case 4: // edge left
        digitalWrite(rWheelDir, LOW);
        wheelMode = 4;
        break;
        
      case 5: // edge right
        digitalWrite(lWheelDir, HIGH);
        wheelMode = 5;
        break;
    }
    
    delay(DELAY);
    
  }
    
  // go steps
  for (unsigned short int index = 0; index < steps; index++) {
    while (micros() - lastStep < SPEED) {} // wait for minimum delay
    
    if (mode != 4) digitalWrite(lWheelStep, HIGH);
    if (mode != 5) digitalWrite(rWheelStep, HIGH);
    
    lastStep = micros();
    delayMicroseconds(10);
    
    if (mode != 4) digitalWrite(lWheelStep, LOW);
    if (mode != 5) digitalWrite(rWheelStep, LOW);
  }
}


void rotate(int steps)
{  
  show_leds(3); // blue
        
  go(2, steps);
  
  show_leds(0); // black

  delay(DELAY);
  
}


int search()
{
  static int smallest;
  static int dir;
  smallest = 10000;
  dir = 0;
  
  show_leds(3); // blue
  
  for (int i = 0; i < ONE_ROUND / 2; i++) { // scan 180 degree
      static int temp;
      
      temp = mSensor.getDist();
      if (temp < smallest) { // remember direction if smallest distance
        smallest = temp;
        dir = i;
      }
      go(3, 1); // rotate 1 step
    }
    
    show_leds(0); // black
    
    delay(DELAY);

    // return direction of object
    return(ONE_ROUND / 2 - dir);
}


bool to_edge()
{
  static unsigned long int start, cantPush;
  cantPush = 0;
  start = millis();
  
  show_leds(2); // green
  
  while (testTable(0)) { // while no edge
    go(0, STEPS); // go steps
    lastStep = mouse.update(lastStep);
    if (200 < millis() - start && !mouse.getY()) { // if not moving
      if (cantPush == 0) { // start timer if not already
        show_leds(1); // red
        cantPush = millis();
      }
    }
    
    else if (cantPush != 0) { // if moving
      cantPush = 0; // reset timer
      show_leds(2); // green
    }
    
    if (cantPush && 500 < millis() - cantPush) { // if not moving for 500 ms
      show_leds(1); // red
      
      delay(DELAY);
      
      show_leds(0); // black

      return false; // return obsticle
    }
  }
  
  show_leds(0); // black
  
  delay(DELAY);
  
  return true; // if edge found return true
}


void push()
{
  show_leds(2); // green
  
  if (testTable(1)) { // edge on left side
    for (int i = 0; i < ONE_ROUND / 2; i++) {
      if (!testTable(1)) { // rotate until edge
        break;
      }
      go(5, 1);
    }
  }
  
  else if (testTable(2)) { // edge on right side
    for (int i = 0; i < ONE_ROUND / 2; i++) {
      if (!testTable(2)) { // rotate until edge
        break;
      }
      go(4, 1);
    }
  }
  
  show_leds(0); // black
  
  delay(DELAY);
  
}


void back_up()
{
  show_leds(4); // white
  
  go(1, ONE_ROUND / 5); // back up a little
  go(3, ONE_ROUND / 4); // rotate 90 degree
  
  show_leds(0); // black
  
  delay(DELAY);
  
}


void show_leds(byte mode)
{
  switch (mode) {
    case 0: // black
      for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = CRGB::Black;
      }
      break;
      
    case 1: // red
      for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = CRGB::Red;
      }
      break;
      
    case 2: // green
      for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = CRGB::Green;
      }
      break;
      
    case 3: // blue
      for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = CRGB::Blue;
      }
      break;
      
    case 4: // white
      for (int i = 0; i < NUM_LEDS; i++) {
        leds[i] = CRGB::White;
      }
      break;
  }
  
  FastLED.show();
}


void loop()
{
  static int dir;
  
  dir = search(); // search for object
  rotate(dir); // face object
  if (to_edge()) { // drive to edge
    push(); // face edge straight
  }
  back_up(); // turn around
}


// [X] = TODO

/*
  Sensors:
    mSensor: mSensor.getDist() // 150 - 1500 (mm)
    lSensor: digitalRead(lSensorRead) // 1 = table, 0 = no table
    rSensor: digitalRead(rSensorRead) // 1 = table, 0 = no table
    xVel: mouse.getX() // 1 = rotating, 0 = not rotating
    yVel: mouse.getY() // 1 = driving, 0 = not driving
    
  Actors:
    lWheelDir: digitalWrite(lWheelDir, <STATE>) // HIGH = forward, LOW = backward
    rWheelDir: digitalWrite(rWheelDir, <STATE>) // HIGH = backward, LOW = forward
    lWheelStep: digitalWrite(lWheelStep, <STATE>) // rising edge = 1/2 step
    rWheelStep: digitalWrite(rWheelStep, <STATE>) // rising edge = 1/2 step
    lights: FastLED.show() // set LEDs acocrding to leds[]
[X] speaker: digitalWrite(speakerPin, <STATE>) // modulate sound

  Functions:
    test table: testTable() // 1 = table ahead, 0 = edge ahead
    go steps: go(steps, mode) // drive for steps, diffrent modes
    rotate: rotate(angle) // rotate for set angle
    search: search() // search for objects, return where to go
    go to edge: to_edge() // returns true if edge, false if obsticle
    push over edge: push() // mode for pushing objects, test if moving, beware edge
    lights: show_leds(<mode>) // show lights according to mode
[X] playJingle: playJingle(<jingle>) // play sounds

*/
